@@ -1,7 +1,11 @@
 use crate::tokenizer::{Decoder, Result};
 
+use serde::{Deserialize, Serialize};
+
+#[derive(Deserialize, Clone, Debug, Serialize)]
 /// The WordPiece decoder takes care of decoding a list of wordpiece tokens
 /// back into a readable string.
+#[serde(tag = "type")]
 pub struct WordPiece {
     /// The prefix to be used for continuing subwords
     prefix: String,
