@@ -2,7 +2,7 @@
 
 use super::{Pair, WithFirstLastIterator, Word, BPE};
 use crate::parallelism::*;
-use crate::tokenizer::{AddedToken, Model, Result, Trainer};
+use crate::tokenizer::{AddedToken, Result, Trainer};
 use indicatif::{ProgressBar, ProgressStyle};
 use std::cmp::Ordering;
 use std::collections::{BinaryHeap, HashMap, HashSet};
@@ -332,7 +332,7 @@ impl BpeTrainer {
                         id2w.push(s.clone());
                         w2id.insert(s.clone(), (id2w.len() - 1) as u32);
                     }
-                    current_word.add(w2id[&s]);
+                    current_word.add(w2id[&s], 1); // We do not care about the len here
                 }
             }
             words.push(current_word);
@@ -555,8 +555,12 @@ impl BpeTrainer {
             word_to_id,
             merges
                 .into_iter()
-                .enumerate()
-                .map(|(index, (pair, new_id))| (pair, (index as u32, new_id)))
+                .map(|((a_id, b_id), _)| {
+                    (
+                        id_to_word[a_id as usize].clone(),
+                        id_to_word[b_id as usize].clone(),
+                    )
+                })
                 .collect(),
         );
         if let Some(prefix) = &self.continuing_subword_prefix {
@@ -575,13 +579,12 @@ impl BpeTrainer {
 }
 
 impl Trainer for BpeTrainer {
+    type Model = BPE;
+
     /// Train a BPE model
-    fn train(
-        &self,
-        word_counts: HashMap<String, u32>,
-    ) -> Result<(Box<dyn Model>, Vec<AddedToken>)> {
+    fn train(&self, word_counts: HashMap<String, u32>) -> Result<(BPE, Vec<AddedToken>)> {
         let (bpe, tokens) = self.train(word_counts)?;
-        Ok((Box::new(bpe), tokens))
+        Ok((bpe, tokens))
     }
 
     /// Process a bunch of tokens, counting them
