@@ -202,11 +202,12 @@ pub fn process_offsets(encoding: &mut Encoding, add_prefix_space: bool) {
 
 #[cfg(test)]
 mod tests {
-    use super::ByteLevel;
+    use super::*;
     use crate::tokenizer::{
         Decoder, Encoding, OffsetReferential, OffsetType, PostProcessor, PreTokenizedString,
         PreTokenizer,
     };
+    use std::iter::FromIterator;
 
     #[test]
     fn pre_tokenization() {
@@ -384,7 +385,7 @@ mod tests {
     #[test]
     fn processor_trims_offsets() {
         let start = Encoding::new(
-            vec![],
+            vec![0; 5],
             vec![],
             vec![
                 "Ġ".into(),
@@ -398,9 +399,10 @@ mod tests {
             vec![],
             vec![],
             vec![],
+            HashMap::new(),
         );
         let expected = Encoding::new(
-            vec![],
+            vec![0; 5],
             vec![],
             vec![
                 "Ġ".into(),
@@ -414,6 +416,7 @@ mod tests {
             vec![],
             vec![],
             vec![],
+            HashMap::new(),
         );
 
         let bytelevel = ByteLevel::default().trim_offsets(true);
@@ -422,7 +425,23 @@ mod tests {
             bytelevel.process(start.clone(), None, false).unwrap()
         );
 
-        let mut pair_expected = expected.clone();
+        let mut pair_expected = Encoding::new(
+            vec![0; 5],
+            vec![],
+            vec![
+                "Ġ".into(),
+                "ĠĠĠĠHelloĠĠ".into(),
+                "ĠĠHello".into(),
+                "HelloĠĠ".into(),
+                "ĠĠĠĠ".into(),
+            ],
+            vec![],
+            vec![(0, 0), (4, 9), (13, 18), (18, 23), (29, 29)],
+            vec![],
+            vec![],
+            vec![],
+            HashMap::from_iter(vec![(0, 0..5), (1, 5..10)]),
+        );
         pair_expected.merge_with(expected, false);
         assert_eq!(
             pair_expected,
