@@ -24,11 +24,6 @@ use serde::de::DeserializeOwned;
 use serde::export::Formatter;
 use serde::{Deserialize, Serialize};
 
-use crate::decoders::DecoderWrapper;
-use crate::models::ModelWrapper;
-use crate::normalizers::NormalizerWrapper;
-use crate::pre_tokenizers::PreTokenizerWrapper;
-use crate::processors::PostProcessorWrapper;
 use crate::utils::parallelism::*;
 
 mod added_vocabulary;
@@ -38,6 +33,13 @@ pub mod pattern;
 pub mod pre_tokenizer;
 mod serialization;
 
+// Re-export wrappers
+pub use crate::decoders::DecoderWrapper;
+pub use crate::models::ModelWrapper;
+pub use crate::normalizers::NormalizerWrapper;
+pub use crate::pre_tokenizers::PreTokenizerWrapper;
+pub use crate::processors::PostProcessorWrapper;
+// And some other types
 pub use crate::utils::iter::LinesWithEnding;
 pub use crate::utils::padding::{pad_encodings, PaddingDirection, PaddingParams, PaddingStrategy};
 pub use crate::utils::truncation::{truncate_encodings, TruncationParams, TruncationStrategy};
@@ -103,7 +105,9 @@ impl dyn PostProcessor {
     ) -> Result<Encoding> {
         match pair_encoding {
             None => Ok(encoding),
-            Some(pair) => {
+            Some(mut pair) => {
+                encoding.set_sequence_id(0);
+                pair.set_sequence_id(1);
                 encoding.merge_with(pair, false);
                 Ok(encoding)
             }
@@ -389,6 +393,14 @@ impl Tokenizer {
     }
 }
 
+impl std::str::FromStr for Tokenizer {
+    type Err = Box<dyn std::error::Error + Send + Sync>;
+
+    fn from_str(s: &str) -> Result<Self> {
+        Ok(serde_json::from_str(s)?)
+    }
+}
+
 impl<M, N, PT, PP, D> From<TokenizerImpl<M, N, PT, PP, D>> for Tokenizer
 where
     M: Into<ModelWrapper>,
