@@ -165,6 +165,7 @@ pub fn truncate_encodings(
 mod tests {
     use super::*;
     use crate::tokenizer::Encoding;
+    use std::collections::HashMap;
 
     fn get_empty() -> Encoding {
         Encoding::new(
@@ -176,6 +177,7 @@ mod tests {
             vec![],
             vec![],
             vec![],
+            HashMap::new(),
         )
     }
 
@@ -189,6 +191,7 @@ mod tests {
             vec![0, 0],
             vec![1, 1],
             vec![],
+            HashMap::new(),
         )
     }
 
@@ -207,6 +210,7 @@ mod tests {
             vec![0, 0, 0, 0],
             vec![1, 1, 1, 1],
             vec![],
+            HashMap::new(),
         )
     }
 
@@ -247,6 +251,7 @@ mod tests {
             vec![0, 0, 0, 0, 0, 0, 0, 0],
             vec![1, 1, 1, 1, 1, 1, 1, 1],
             vec![],
+            HashMap::new(),
         )
     }
 
